@@ -1,7 +1,7 @@
 #!/usr/bin/env python
 # -*- coding: utf-8 -*-
 
-# Copyright (c) 2010, 2011, 2012, 2013
+# Copyright (c) 2010, 2011, 2012, 2013, 2014
 
 # Author(s):
 
@@ -27,7 +27,7 @@
 """
 #raise NotImplementedError
 import logging
-logger = logging.getLogger('mipp')
+logger = logging.getLogger(__name__)
 
 import sys
 import numpy as np
@@ -46,7 +46,7 @@ no_data_value = 0
 
 if sys.version_info[0] >= 2 and sys.version_info[1] >= 5:
     try:
-        #Use numexpr if available
+        # Use numexpr if available
         import numexpr
         eval_np = numexpr.evaluate
         logger.info('Using numexpr for fast numpy evaluation')
@@ -55,23 +55,24 @@ if sys.version_info[0] >= 2 and sys.version_info[1] >= 5:
 else:
     logger.warning('Older version of python. Module numexpr not used. '
                    'Performance will be slower.')
-        
 
-#Reflectance factor for visible bands
-HRV_F    = 25.15
+
+# Reflectance factor for visible bands
+HRV_F = 25.15
 VIS006_F = 20.76
 VIS008_F = 23.30
 IR_016_F = 19.73
 
-## Calibration coefficients from
-##'A Planned Change to the MSG Level 1.5 Image Product Radiance Definition'
-## ,
-## "Conversion from radiances to reflectances for SEVIRI warm channels"
-## EUM/MET/TEN/12/0332
-## , and
-## "The Conversion from Effective Radiances to Equivalent Brightness
-## Temperatures"
-## EUM/MET/TEN/11/0569
+SATNUM = {321: "08",
+          322: "09",
+          323: "10",
+          324: "11"}
+
+# Calibration coefficients from
+# 'A Planned Change to the MSG Level 1.5 Image Product Radiance Definition',
+# "Conversion from radiances to reflectances for SEVIRI warm channels"
+# EUM/MET/TEN/12/0332 , and "The Conversion from Effective Radiances to
+# Equivalent Brightness Temperatures" EUM/MET/TEN/11/0569
 
 CALIB = {}
 
@@ -171,7 +172,7 @@ CALIB[323] = {'HRV': {'F': 78.9416 / np.pi},
 
 # Meteosat 11
 
-CALIB[324] = {'HRV': {'F': 79.0035/ np.pi},
+CALIB[324] = {'HRV': {'F': 79.0035 / np.pi},
               'VIS006': {'F': 65.2656 / np.pi},
               'VIS008': {'F': 73.1692 / np.pi},
               'IR_016': {'F': 61.9416 / np.pi},
@@ -200,43 +201,46 @@ CALIB[324] = {'HRV': {'F': 79.0035/ np.pi},
                          'ALPHA': 0.9981,
                          'BETA': 0.5635}}
 
-#Polynomial coefficients for spectral-effective BT fits
-BTFIT_A_IR_039 =  0.0
-BTFIT_A_WV_062 =  0.00001805700
-BTFIT_A_WV_073 =  0.00000231818
+# Polynomial coefficients for spectral-effective BT fits
+BTFIT_A_IR_039 = 0.0
+BTFIT_A_WV_062 = 0.00001805700
+BTFIT_A_WV_073 = 0.00000231818
 BTFIT_A_IR_087 = -0.00002332000
-BTFIT_A_IR_097 = -0.00002055330 
+BTFIT_A_IR_097 = -0.00002055330
 BTFIT_A_IR_108 = -0.00007392770
 BTFIT_A_IR_120 = -0.00007009840
 BTFIT_A_IR_134 = -0.00007293450
 
-BTFIT_B_IR_039 =  1.011751900
-BTFIT_B_WV_062 =  1.000255533 
-BTFIT_B_WV_073 =  1.000668281
-BTFIT_B_IR_087 =  1.011803400
-BTFIT_B_IR_097 =  1.009370670  
-BTFIT_B_IR_108 =  1.032889800 
-BTFIT_B_IR_120 =  1.031314600
-BTFIT_B_IR_134 =  1.030424800
+BTFIT_B_IR_039 = 1.011751900
+BTFIT_B_WV_062 = 1.000255533
+BTFIT_B_WV_073 = 1.000668281
+BTFIT_B_IR_087 = 1.011803400
+BTFIT_B_IR_097 = 1.009370670
+BTFIT_B_IR_108 = 1.032889800
+BTFIT_B_IR_120 = 1.031314600
+BTFIT_B_IR_134 = 1.030424800
 
-BTFIT_C_IR_039 =  -3.550400
-BTFIT_C_WV_062 =  -1.790930
-BTFIT_C_WV_073 =  -0.456166
-BTFIT_C_IR_087 =  -1.507390
-BTFIT_C_IR_097 =  -1.030600
-BTFIT_C_IR_108 =  -3.296740
-BTFIT_C_IR_120 =  -3.181090
-BTFIT_C_IR_134 =  -2.645950
+BTFIT_C_IR_039 = -3.550400
+BTFIT_C_WV_062 = -1.790930
+BTFIT_C_WV_073 = -0.456166
+BTFIT_C_IR_087 = -1.507390
+BTFIT_C_IR_097 = -1.030600
+BTFIT_C_IR_108 = -3.296740
+BTFIT_C_IR_120 = -3.181090
+BTFIT_C_IR_134 = -2.645950
 
 
 C1 = 1.19104273e-16
 C2 = 0.0143877523
 
+
 class _Calibrator(object):
-    def __init__(self, hdr, channel_name):
+
+    def __init__(self, hdr, channel_name, bits_per_pixel):
         self.hdr = hdr
         self.channel_name = channel_name
-        
+        self.bits_per_pixel = bits_per_pixel
+
     def __call__(self, image, calibrate=1):
         """Computes the radiances and reflectances/bt of a given channel.  The
         *calibrate* argument should be set to 0 for no calibration, 1 for
@@ -248,7 +252,7 @@ class _Calibrator(object):
         channel_name = self.channel_name
 
         if calibrate == 0:
-            return (image, 
+            return (image,
                     "counts")
 
         channels = {"VIS006": 1,
@@ -268,18 +272,20 @@ class _Calibrator(object):
         chn_nb = channels[channel_name] - 1
 
         mask = (image == no_data_value)
-        
+        if self.bits_per_pixel == 8:
+            logger.info("8 bits per pixel adjusted to 10 bits for calibration.")
+            image = image.astype(np.uint16) * 4
+
         cslope = hdr["Level1_5ImageCalibration"][chn_nb]['Cal_Slope']
         coffset = hdr["Level1_5ImageCalibration"][chn_nb]['Cal_Offset']
-        
+
         radiances = eval_np('image * cslope + coffset')
         radiances[radiances < 0] = 0
-        
+
         if calibrate == 2:
             return (np.ma.MaskedArray(radiances, mask=mask),
                     "mW m-2 sr-1 (cm-1)-1")
-            
-        
+
         sat = hdr["SatelliteDefinition"]["SatelliteId"]
         if sat not in CALIB:
             raise CalibrationError("No calibration coefficients available for "
@@ -288,30 +294,30 @@ class _Calibrator(object):
         if channel_name in ["HRV", "VIS006", "VIS008", "IR_016"]:
             solar_irradiance = CALIB[sat][channel_name]["F"]
             reflectance = eval_np('(radiances / solar_irradiance) * 100.')
-            return (np.ma.MaskedArray(reflectance, mask=mask), 
+            return (np.ma.MaskedArray(reflectance, mask=mask),
                     "%")
 
         wavenumber = CALIB[sat][channel_name]["VC"]
         if cal_type[chn_nb] == 2:
-            #computation based on effective radiance
+            # computation based on effective radiance
             alpha = CALIB[sat][channel_name]["ALPHA"]
             beta = CALIB[sat][channel_name]["BETA"]
-            
-            cal_data = eval_np(('((C2 * 100. * wavenumber / ' 
-                                'log(C1 * 1.0e6 * wavenumber ** 3 / ' 
+
+            cal_data = eval_np(('((C2 * 100. * wavenumber / '
+                                'log(C1 * 1.0e6 * wavenumber ** 3 / '
                                 '(1.0e-5 * radiances) + 1)) - beta) / alpha'))
-            
+
         elif cal_type[chn_nb] == 1:
-            #computation based on spectral radiance
+            # computation based on spectral radiance
             cal_data = eval_np(('C2 * 100. * wavenumber / '
                                 'log(C1 * 1.0e6 * wavenumber ** 3 / '
                                 '(1.0e-5 * radiances) + 1))'))
-            
+
             coef_a = eval("BTFIT_A_" + channel_name)
             coef_b = eval("BTFIT_B_" + channel_name)
             coef_c = eval("BTFIT_C_" + channel_name)
-            
-            cal_data = eval_np(('cal_data ** 2 * coef_a + ' 
+
+            cal_data = eval_np(('cal_data ** 2 * coef_a + '
                                 'cal_data * coef_b + coef_c'))
 
         else:
@@ -319,9 +325,10 @@ class _Calibrator(object):
 
         mask = mask | np.isnan(cal_data) | np.isinf(cal_data)
         cal_data = np.ma.MaskedArray(cal_data, mask=mask)
-        return (cal_data, 
+        return (cal_data,
                 "K")
 
+
 def read_proheader(fp):
     """Read the msg header.
     """
@@ -343,11 +350,11 @@ def read_proheader(fp):
     satop["LastManoeuvreFlag"] = ord(fp.read(1)) > 0
     satop["LastManoeuvreStartTime"] = rbin.read_cds_time(fp.read(6))
     satop["LastManoeuvreEndTime"] = rbin.read_cds_time(fp.read(6))
-    satop["LastManoeuvreType"] =  ord(fp.read(1))
+    satop["LastManoeuvreType"] = ord(fp.read(1))
     satop["NextManoeuvreFlag"] = ord(fp.read(1)) > 0
     satop["NextManoeuvreStartTime"] = rbin.read_cds_time(fp.read(6))
     satop["NextManoeuvreEndTime"] = rbin.read_cds_time(fp.read(6))
-    satop["NextManoeuvreType"] =  ord(fp.read(1))
+    satop["NextManoeuvreType"] = ord(fp.read(1))
 
     hdr["SatelliteOperations"] = satop
     del satop
@@ -371,7 +378,7 @@ def read_proheader(fp):
 
     attitude = {}
     attitude["PeriodStartTime"] = rbin.read_cds_time(fp.read(6))
-    attitude["PeriodEndTime"] =  rbin.read_cds_time(fp.read(6))
+    attitude["PeriodEndTime"] = rbin.read_cds_time(fp.read(6))
     attitude["PrincipleAxisOffsetAngle"] = rbin.read_float8(fp.read(8))
     attitudecoef = np.dtype(">u2, >u4, >u2, >u4, (8,)>f8, (8,)>f8, (8,)>f8")
     attitude["AttitudePolynomial"] = np.fromstring(fp.read(20400),
@@ -380,15 +387,15 @@ def read_proheader(fp):
 
     hdr["Attitude"] = attitude
     del attitude
-    
+
     # SpinRateatRCStart
-    
+
     hdr["SpinRateatRCStart"] = rbin.read_float8(fp.read(8))
 
     # UTCCorrelation
 
     utccor = {}
-    
+
     utccor["PeriodStartTime"] = rbin.read_cds_time(fp.read(6))
     utccor["PeriodEndTime"] = rbin.read_cds_time(fp.read(6))
     utccor["OnBoardTimeStart"] = rbin.read_cuc_time(fp.read(7), 4, 3)
@@ -411,7 +418,7 @@ def read_proheader(fp):
     hdr["PlannedAcquisitionTime"] = pat
 
     # RadiometerStatus
-    
+
     radiostatus = {}
     radiostatus["ChannelStatus"] = np.fromstring(fp.read(12), dtype=np.uint8)
     radiostatus["DetectorStatus"] = np.fromstring(fp.read(42), dtype=np.uint8)
@@ -466,7 +473,6 @@ def read_proheader(fp):
     radiooper["Decontamination"]["DecontaminationStart"] = rbin.read_cds_time(fp.read(6))
     radiooper["Decontamination"]["DecontaminationEnd"] = rbin.read_cds_time(fp.read(6))
 
-
     radiooper["BBCalScheduled"] = ord(fp.read(1)) > 0
     radiooper["BBCalibrationType"] = ord(fp.read(1))
     radiooper["BBFirstLine"] = rbin.read_uint2(fp.read(2))
@@ -474,10 +480,9 @@ def read_proheader(fp):
     radiooper["ColdFocalPlaneOpTemp"] = rbin.read_uint2(fp.read(2))
     radiooper["WarmFocalPlaneOpTemp"] = rbin.read_uint2(fp.read(2))
 
-
     hdr["RadiometerOperations"] = radiooper
 
-    ## CelestialEvents
+    # CelestialEvents
     # CelestialBodiesPosition
 
     celbodies = {}
@@ -487,14 +492,14 @@ def read_proheader(fp):
     celbodies["RelatedAttitudeFileTime"] = fp.read(15)
     earthmoonsuncoef = np.dtype(">u2, >u4, >u2, >u4, (8,)>f8, (8,)>f8")
     celbodies["EarthEphemeris"] = np.fromstring(fp.read(14000),
-                                             dtype=earthmoonsuncoef,
-                                             count=100)
+                                                dtype=earthmoonsuncoef,
+                                                count=100)
     celbodies["MoonEphemeris"] = np.fromstring(fp.read(14000),
-                                             dtype=earthmoonsuncoef,
-                                             count=100)
+                                               dtype=earthmoonsuncoef,
+                                               count=100)
     celbodies["SunEphemeris"] = np.fromstring(fp.read(14000),
-                                             dtype=earthmoonsuncoef,
-                                             count=100)
+                                              dtype=earthmoonsuncoef,
+                                              count=100)
     starcoef = np.dtype(">u2, >u2, >u4, >u2, >u4, (8,)>f8, (8,)>f8")
     starcoefs = np.dtype([('starcoefs', starcoef, (20,))])
 
@@ -516,7 +521,7 @@ def read_proheader(fp):
 
     hdr["RelationToImage"] = reltoim
 
-    ## ImageDescriptionRecord
+    # ImageDescriptionRecord
 
     grid_origin = ["north west", "south west", "south east", "north east"]
 
@@ -559,11 +564,11 @@ def read_proheader(fp):
     covvisir["WesternColumnPlanned"] = rbin.read_int4(fp.read(4))
 
     hdr["PlannedCoverageVIS_IR"] = covvisir
-    
+
     # PlannedCoverageHRV
 
     covhrv = {}
-    
+
     covhrv["LowerSouthLinePlanned"] = rbin.read_int4(fp.read(4))
     covhrv["LowerNorthLinePlanned"] = rbin.read_int4(fp.read(4))
     covhrv["LowerEastColumnPlanned"] = rbin.read_int4(fp.read(4))
@@ -579,7 +584,7 @@ def read_proheader(fp):
 
     image_proc_direction = ["North-South", "South-North"]
     pixel_gen_direction = ["East-West", "West-East"]
-    
+
     l15prod = {}
     l15prod["ImageProcDirection"] = image_proc_direction[ord(fp.read(1))]
     l15prod["PixelGenDirection"] = pixel_gen_direction[ord(fp.read(1))]
@@ -590,14 +595,13 @@ def read_proheader(fp):
 
     hdr["Level 1_5 ImageProduction"] = l15prod
 
-
-    ## RadiometricProcessing
+    # RadiometricProcessing
 
     # RPSummary
 
     rpsummary = {}
     rpsummary["RadianceLinearization"] = np.fromstring(fp.read(12), dtype=np.bool)
-    
+
     rpsummary["DetectorEqualization"] = np.fromstring(fp.read(12), dtype=np.bool)
     rpsummary["OnboardCalibrationResult"] = np.fromstring(fp.read(12), dtype=np.bool)
     rpsummary["MPEFCalFeedback"] = np.fromstring(fp.read(12), dtype=np.bool)
@@ -612,7 +616,6 @@ def read_proheader(fp):
 
     hdr["Level1_5ImageCalibration"] = np.fromstring(fp.read(192), dtype=caltype)
 
-
     # BlackBodyDataUsed
 
     bbdu = {}
@@ -623,24 +626,24 @@ def read_proheader(fp):
     bbdu["BBRelatedData"]["MDUOutGain"] = np.fromstring(fp.read(42 * 2),
                                                         dtype=">u2")
     bbdu["BBRelatedData"]["MDUCoarseGain"] = np.fromstring(fp.read(42),
-                                                        dtype=np.uint8)
+                                                           dtype=np.uint8)
     bbdu["BBRelatedData"]["MDUFineGain"] = np.fromstring(fp.read(42 * 2),
-                                                        dtype=">u2")
+                                                         dtype=">u2")
     bbdu["BBRelatedData"]["MDUNumericalOffset"] = np.fromstring(fp.read(42 * 2),
-                                                        dtype=">u2")
+                                                                dtype=">u2")
     bbdu["BBRelatedData"]["PUGain"] = np.fromstring(fp.read(42 * 2),
-                                                        dtype=">u2")
+                                                    dtype=">u2")
     bbdu["BBRelatedData"]["PUOffset"] = np.fromstring(fp.read(27 * 2),
-                                                        dtype=">u2")
+                                                      dtype=">u2")
     bbdu["BBRelatedData"]["PUBias"] = np.fromstring(fp.read(15 * 2),
-                                                        dtype=">u2")
+                                                    dtype=">u2")
     # 12 bits bitstrings... convert to uint16
     data = np.fromstring(fp.read(int(42 * 1.5)),
                          dtype=np.uint8)
     data = data.astype(np.uint16)
-    data[::3] = data[::3]*256 + data[1::3] // 16
-    data[1::3] = (data[1::3] & 0x0f)*16 + data[2::3]
-    result = np.ravel(data.reshape(-1,3)[:,:2])
+    data[::3] = data[::3] * 256 + data[1::3] // 16
+    data[1::3] = (data[1::3] & 0x0f) * 16 + data[2::3]
+    result = np.ravel(data.reshape(-1, 3)[:, :2])
     bbdu["BBRelatedData"]["DCRValues"] = result
     bbdu["BBRelatedData"]["X_DeepSpaceWindowPosition"] = ord(fp.read(1))
     bbdu["BBRelatedData"]["ColdFPTemperature"] = {}
@@ -679,7 +682,7 @@ def read_proheader(fp):
                                     ('MinCount', '>u2'),
                                     ('BB_Processing_Slope', '>f8'),
                                     ('BB_Processing_Offset', '>f8')])
-    
+
     bbdu["BBRelatedData"]["ExtractedBBData"] = np.fromstring(fp.read(32 * 12),
                                                              dtype=extracted_data_type)
     impf_cal_type = np.dtype([("ImageQualityFlag", "u1"),
@@ -690,18 +693,17 @@ def read_proheader(fp):
                               ("AbsCalWeightXsat", ">f4"),
                               ("AbsCalCoeff", ">f4"),
                               ("AbsCalError", ">f4"),
-                              ("CalMonBias", ">f4"),
-                              ("CalMonRms", ">f4"),
-                              ("OffsetCount", ">f4")])
+                              ("GSICSCalCoeff", ">f4"),
+                              ("GSICSCalError", ">f4"),
+                              ("GSICSOffsetCount", ">f4")])
 
-    
     bbdu["MPEFCalFeedback"] = np.fromstring(fp.read(32 * 12),
                                             dtype=impf_cal_type)
-    
+
     bbdu["RadTransform"] = np.fromstring(fp.read(42 * 64 * 4),
-                                            dtype=">f4").reshape((42,64))
+                                         dtype=">f4").reshape((42, 64))
     bbdu["RadProcMTFAdaptation"] = {}
-    
+
     bbdu["RadProcMTFAdaptation"]["VIS_IRMTFCorrectionE_W"] = np.fromstring(fp.read(33 * 16 * 4),
                                                                            dtype=">f4").reshape((33, 16))
     bbdu["RadProcMTFAdaptation"]["VIS_IRMTFCorrectionN_S"] = np.fromstring(fp.read(33 * 16 * 4),
@@ -711,7 +713,7 @@ def read_proheader(fp):
     bbdu["RadProcMTFAdaptation"]["HRVMTFCorrectionN_S"] = np.fromstring(fp.read(9 * 16 * 4),
                                                                         dtype=">f4").reshape((9, 16))
     bbdu["RadProcMTFAdaptation"]["StraylightCorrection"] = np.fromstring(fp.read(12 * 8 * 8 * 4),
-                                                                        dtype=">f4").reshape((12, 8, 8))
+                                                                         dtype=">f4").reshape((12, 8, 8))
 
     hdr["BlackBodyDataUsed"] = bbdu
 
@@ -738,6 +740,7 @@ def read_proheader(fp):
 
     return hdr
 
+
 def read_epiheader(fp):
     """Read the msg header.
     """
@@ -765,9 +768,9 @@ def read_epiheader(fp):
     ftr["NumberOfMissingL10Lines"] = np.fromstring(fp.read(12 * 4),
                                                    dtype=">u4")
     ftr["NumberOfCorruptedL10Lines"] = np.fromstring(fp.read(12 * 4),
-                                                   dtype=">u4")
+                                                     dtype=">u4")
     ftr["NumberOfReplacedL10Lines"] = np.fromstring(fp.read(12 * 4),
-                                                   dtype=">u4")
+                                                    dtype=">u4")
     validitytype = np.dtype([('NominalImage', '>u1'),
                              ('NonNominalBecauseIncomplete', '>u1'),
                              ('NonNominalRadiometricQuality', '>u1'),
@@ -792,21 +795,22 @@ def read_epiheader(fp):
 
     return ftr
 
+
 def read_metadata(prologue, image_files, epilogue):
-    """ Selected items from the Meteosat-9 prolog file.
+    """ Selected items from the MSG prologue file.
     """
-    segment_size = 464 # number of lines in a segment
+    segment_size = 464  # number of lines in a segment
 
     fp = StringIO(prologue.data)
     hdr = read_proheader(fp)
 
     fp = StringIO(epilogue.data)
     ftr = read_epiheader(fp)
-    
+
     im = _xrit.read_imagedata(image_files[0])
 
     md = Metadata()
-    md.calibrate = _Calibrator(hdr, im.product_name)
+    md.calibrate = _Calibrator(hdr, im.product_name, bits_per_pixel=im.structure.nb)
 
     md.sublon = hdr["ProjectionDescription"]["LongitudeOfSSP"]
     md.product_name = im.product_id
@@ -817,8 +821,10 @@ def read_metadata(prologue, image_files, epilogue):
     else:
         md.image_size = np.array((hdr["ReferenceGridVIS_IR"]["NumberOfLines"],
                                   hdr["ReferenceGridVIS_IR"]["NumberOfColumns"]))
-        
+
     md.satname = im.platform.lower()
+    md.satnumber = SATNUM[hdr["SatelliteDefinition"]["SatelliteId"]]
+    logger.debug("%s %s", md.satname, md.satnumber)
     md.product_type = 'full disc'
     md.region_name = 'full disc'
     if md.channel == "HRV":
@@ -829,16 +835,14 @@ def read_metadata(prologue, image_files, epilogue):
             ftr["LowerNorthLineActual"],
             ftr["LowerEastColumnActual"],
             ftr["LowerWestColumnActual"]],
-           [ftr["UpperSouthLineActual"],
-            ftr["UpperNorthLineActual"],
-            ftr["UpperEastColumnActual"],
-            ftr["UpperWestColumnActual"]]])
+            [ftr["UpperSouthLineActual"],
+             ftr["UpperNorthLineActual"],
+             ftr["UpperEastColumnActual"],
+             ftr["UpperWestColumnActual"]]])
 
-        im_loff = im.navigation.loff + segment_size * (im.segment.seg_no - 1)
-        md.coff = (ftr["Lower"+ew_.capitalize()+"ColumnActual"]
-                   + im.navigation.coff - 1)
-        md.loff = (ftr["Lower"+ns_.capitalize()+"LineActual"]
-                   + im_loff - 1)
+        md.coff = (ftr["Lower" + ew_.capitalize() + "ColumnActual"]
+                   + im.navigation.coff + 1)
+        md.loff = im.navigation.loff + segment_size * (im.segment.seg_no - 1) + 2
 
     else:
         md.first_pixel = hdr["ReferenceGridVIS_IR"]["GridOrigin"]
@@ -849,21 +853,43 @@ def read_metadata(prologue, image_files, epilogue):
             ftr["EasternColumnActual"],
             ftr["WesternColumnActual"]]])
 
-        im_loff = im.navigation.loff + segment_size * (im.segment.seg_no - 1)
-        md.coff = (ftr[ew_.capitalize()+"ernColumnActual"]
-                   + im.navigation.coff - 1)
-        md.loff = (ftr[ns_.capitalize()+"ernLineActual"]
-                   + im_loff - 1)
+        md.coff = im.navigation.coff
+        md.loff = im.navigation.loff + segment_size * (im.segment.seg_no - 1)
 
     md.data_type = im.structure.nb
     md.no_data_value = no_data_value
     md.line_offset = 0
     md.time_stamp = im.time_stamp
     md.production_time = im.production_time
-    md.calibration_unit = 'counts'
+    md.calibration_unit = ""
 
     return md
 
+
+def read_obstimes(epilogue):
+    """Get the start and end full disk scan times from the Epilogue file."""
+    epi = _xrit.read_epilogue(epilogue)
+    fpt = StringIO(epi.data)
+    ftr = read_epiheader(fpt)
+    return (ftr['ForwardScanStart'], ftr['ForwardScanEnd'])
+
+
+def read_scanline_quality(segment_filename):
+    """Get the line quality data from the segment file. This is from the
+    ImageSegmentLineQuality class of the _xrit module, which maps the Header
+    Type 129 - Image Segment Line Quality as decribed in the 'MSG Ground
+    Segment LRIT/HRIT Mission Specific Implementation, EUM/MSG/SPE/057'
+    document, see reference below.
+
+    This record contains the scan line mean acquisition times
+
+    http://www.eumetsat.int/website/wcm/idc/idcplg?IdcService=GET_FILE&dDocName=PDF_TEN_05057_SPE_MSG_LRIT_HRI&RevisionSelectionMethod=LatestReleased&Rendition=Web
+
+    """
+    imgdata = _xrit.read_imagedata(segment_filename)
+    return imgdata.image_quality.line_quality
+
+
 if __name__ == '__main__':
     p = _xrit.read_prologue(sys.argv[1])
     e = _xrit.read_epilogue(sys.argv[-1])
